@@ -22,16 +22,19 @@
 #include "config.h"
 #endif
 #include "first.h"
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #define GIAC_CONTEXT const context * contextptr
 #define GIAC_CONTEXT0 const context * contextptr=0
 
-#if !defined(HAVE_NO_SYS_TIMES_H) && !defined(BESTA_OS) && !defined(__MINGW_H) && !defined(NSPIRE)
+#if !defined(HAVE_NO_SYS_TIMES_H) && !defined(BESTA_OS) && !defined(__MINGW_H) && !defined(NSPIRE) && !defined(FXCG)
 #include <sys/times.h>
 #else
-#if defined VISUALC || defined BESTA_OS 
+#if defined VISUALC || defined BESTA_OS || defined FREERTOS
 typedef long pid_t;
 #else // VISUALC
-#if !defined(__MINGW_H) && !defined(NSPIRE) && !defined(__ANDROID__) && !defined(NSPIRE_NEWLIB) && !defined(OSX) && !defined(IOS) && !defined(OSXIOS) && !defined(FIR_LINUX)
+#if !defined(__MINGW_H) && !defined(NSPIRE) && !defined(FXCG) && !defined(__ANDROID__) && !defined(NSPIRE_NEWLIB) && !defined(OSX) && !defined(IOS) && !defined(OSXIOS) && !defined(FIR_LINUX) && !defined(PRIMEWEBASM)
 #include "wince_replacements.h"
 #endif
 #ifdef __MINGW_H
@@ -77,7 +80,7 @@ inline double giac_log(double d){
 
 #include "vector.h"
 #include <string>
-#ifndef NSPIRE
+#if !defined( NSPIRE) && !defined(FXCG)
 #include <cstring>
 #endif
 #include <iostream>
@@ -94,7 +97,7 @@ inline double giac_log(double d){
 
 #include <stdexcept>
 #include "help.h"
-#if 1 // !defined(GIAC_HAS_STO_38) // && !defined(ConnectivityKit) && !defined(BESTA_OS)
+#if !defined(FXCG) // !defined(GIAC_HAS_STO_38) // && !defined(ConnectivityKit) && !defined(BESTA_OS)
 #include "tinymt32.h"
 #endif
 
@@ -199,7 +202,7 @@ Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd);
   std::vector<aide> * & vector_aide_ptr();
   std::vector<std::string> * & vector_completions_ptr();
   extern void (*fl_widget_delete_function)(void *);
-#ifndef NSPIRE
+#if !defined( NSPIRE) && !defined(FXCG)
   extern std::ostream & (*fl_widget_archive_function)(std::ostream &,void *);
 #endif
   extern bool secure_run; // true if used in a non-trusted environment
@@ -265,6 +268,10 @@ Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd);
   extern int GBASISF4_MAX_TOTALDEG;
   extern int GBASISF4_MAXITER;
   // extern int GBASISF4_BUCHBERGER;
+  extern unsigned max_pairs_by_iteration; 
+  extern unsigned simult_primes; 
+  extern double gbasis_reinject_ratio;
+  extern double gbasis_reinject_speed_ratio;
   extern int PROOT_FACTOR_MAXDEG;
   extern int ABS_NBITS_EVALF;
   extern volatile bool ctrl_c,interrupted;
@@ -287,7 +294,7 @@ Boolean isLegalUTF8Sequence(const UTF8 *source, const UTF8 *sourceEnd);
 
   // void control_c();
   // note that ctrl_c=false was removed, should be done before calling eval
-#if defined NSPIRE
+#if defined (NSPIRE) || defined(FXCG)
   void control_c();
 #elif defined FIR
 #define control_c()
@@ -444,7 +451,7 @@ throw(std::runtime_error("Stopped by user interruption.")); \
     thread_param();
   };
 
-#ifndef NSPIRE
+#if !defined( NSPIRE) && !defined(FXCG)
   extern gen (*fl_widget_unarchive_function)(std::istream &);
 #endif
   extern std::string (*fl_widget_texprint_function)(void * ptr);
@@ -511,6 +518,7 @@ throw(std::runtime_error("Stopped by user interruption.")); \
     int _xcas_mode_;
     int _calc_mode_;
     int _decimal_digits_;
+    int _minchar_for_quote_as_string_;
     int _scientific_format_;
     int _integer_format_;
     int _latex_format_;
@@ -570,7 +578,7 @@ throw(std::runtime_error("Stopped by user interruption.")); \
     parser_lexer _pl;
     int _prog_eval_level_val ;
     int _eval_level;
-#if 0 // defined(GIAC_HAS_STO_38) || defined(ConnectivityKit)
+#if defined(FXCG) // defined(GIAC_HAS_STO_38) || defined(ConnectivityKit)
     unsigned int _rand_seed;
 #else
     tinymt32_t _rand_seed;
@@ -580,8 +588,8 @@ throw(std::runtime_error("Stopped by user interruption.")); \
     pthread_mutex_t * _mutexptr,* _mutex_eval_status_ptr ;
 #endif
     int _language_;
-    std::vector<const char *> _last_evaled_function_name_;
-    vecteur _last_evaled_arg_;
+    const char * _last_evaled_function_name_;
+    const gen * _last_evaled_argptr_;
     int _max_sum_sqrt_;
     int _max_sum_add_;
     logo_turtle _turtle_;
@@ -589,6 +597,7 @@ throw(std::runtime_error("Stopped by user interruption.")); \
     std::string _format_double_;
     std::string _autosimplify_;
     std::string _lastprog_name_;
+    std::string _currently_scanned_;
     std::vector<logo_turtle> _turtle_stack_; 
     double _total_time_;
     void * _evaled_table_;
@@ -615,7 +624,7 @@ throw(std::runtime_error("Stopped by user interruption.")); \
     global * globalptr; 
     const context * parent;
     vecteur * quoted_global_vars, * rootofs;
-    vecteur * history_in_ptr, * history_out_ptr;
+    vecteur * history_in_ptr, * history_out_ptr,*history_plot_ptr;
     context();
     context(const context & c);
 #ifndef RTOS_THREADX
@@ -636,12 +645,13 @@ throw(std::runtime_error("Stopped by user interruption.")); \
   extern pthread_mutex_t context_list_mutex;
 #endif
   
-#if !defined(RTOS_THREADX) && !defined(BESTA_OS) && !defined(NSPIRE)
+#if !defined(RTOS_THREADX) && !defined(BESTA_OS) && !defined(NSPIRE) && !defined(FXCG)
   extern std::map<std::string,context *> * context_names ;
 #endif
 
-  std::vector<const char *> & last_evaled_function_name(GIAC_CONTEXT);
-  vecteur & last_evaled_arg(GIAC_CONTEXT);
+  const char * & last_evaled_function_name(GIAC_CONTEXT);
+  std::string & currently_scanned(GIAC_CONTEXT);
+  const gen * & last_evaled_argptr(GIAC_CONTEXT);
 
   bool make_thread(const giac::gen & g,int level,const giac_callback & f,void * f_param,const context * contextptr);
 
@@ -666,6 +676,7 @@ throw(std::runtime_error("Stopped by user interruption.")); \
   int & python_compat(GIAC_CONTEXT);
   void python_compat(int b,GIAC_CONTEXT);
   int array_start(GIAC_CONTEXT);
+  extern bool python_color; // global variable for syntax highlighting
 
   int & calc_mode(GIAC_CONTEXT);
   int abs_calc_mode(GIAC_CONTEXT);
@@ -677,6 +688,9 @@ throw(std::runtime_error("Stopped by user interruption.")); \
   int & decimal_digits(GIAC_CONTEXT);
   void decimal_digits(int b,GIAC_CONTEXT);
 
+  int & minchar_for_quote_as_string(GIAC_CONTEXT);
+  void minchar_for_quote_as_string(int b,GIAC_CONTEXT);
+
   int & integer_format(GIAC_CONTEXT);
   void integer_format(int b,GIAC_CONTEXT);
   int & latex_format(GIAC_CONTEXT);
@@ -739,6 +753,7 @@ throw(std::runtime_error("Stopped by user interruption.")); \
 
   vecteur & history_in(GIAC_CONTEXT);
   vecteur & history_out(GIAC_CONTEXT);
+  vecteur & history_plot(GIAC_CONTEXT);
 
   // True if we factor 2nd order polynomials using sqrt
   bool & withsqrt(GIAC_CONTEXT);
@@ -826,7 +841,7 @@ throw(std::runtime_error("Stopped by user interruption.")); \
   // void eval_level(int b,GIAC_CONTEXT);
   thread_param * thread_param_ptr(const context * contextptr);
 
-#if 0 // defined(GIAC_HAS_STO_38) || defined(ConnectivityKit)
+#if defined(FXCG) // defined(GIAC_HAS_STO_38) || defined(ConnectivityKit)
   unsigned int & rand_seed(GIAC_CONTEXT);
 #else
   tinymt32_t * rand_seed(GIAC_CONTEXT);
@@ -929,6 +944,8 @@ throw(std::runtime_error("Stopped by user interruption.")); \
   int equalposcomp(const std::vector<int> v,int i);
   int equalposcomp(const std::vector<short int> v,int i);
   int equalposcomp(int tab[],int f);
+  // replace c1 by c2 in s
+  std::string replace(const std::string & s,char c1,char c2);
   // attempt to convert Python-like programming structures to Xcas
   std::string python2xcas(const std::string & s_orig,GIAC_CONTEXT);
   std::string find_doc_prefix(int i);
